@@ -45,19 +45,20 @@
 
 Render_t                                render;
 int                                     eurAscii = -1;
+int                                     aspect_ratio_x = 0;
+int                                     aspect_ratio_y = 0;
 Bitu                                    last_gfx_flags = 0;
 ScalerLineHandler_t                     RENDER_DrawLine;
 
 uint32_t                                GFX_palette32bpp[256] = {0};
 
 unsigned int                            GFX_GetBShift();
+void                                    aspect_ratio_menu();
 void                                    RENDER_CallBack( GFX_CallBackFunctions_t function );
 bool                                    systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
 
 #if defined(USE_TTF)
 bool resetreq=false;
-ttf_cell curAttrChar[txtMaxLins*txtMaxCols];					// currently displayed textpage
-ttf_cell newAttrChar[txtMaxLins*txtMaxCols];					// to be replaced by
 void resetFontSize();
 #endif
 
@@ -365,7 +366,15 @@ extern Bitu PIC_Ticks;
 extern bool pause_on_vsync;
 extern bool checkmenuwidth;
 void PauseDOSBox(bool pressed);
-void AspectRatio_mapper_shortcut(bool pressed);
+bool GFX_GetPreventFullscreen(void);
+
+void AspectRatio_mapper_shortcut(bool pressed) {
+    if (!pressed) return;
+
+    if (!GFX_GetPreventFullscreen()) {
+        SetVal("render", "aspect", render.aspect ? "false" : "true");
+    }
+}
 
 void RENDER_EndUpdate( bool abort ) {
     if (GCC_UNLIKELY(!render.updating))
@@ -720,8 +729,8 @@ forcenormal:
         }
     }
 /* update the aspect ratio */
-    sdl.srcAspect.x = (int)(render.src.width * (render.src.dblw ? 2 : 1));
-    sdl.srcAspect.y = (int)floor((render.src.height * (render.src.dblh ? 2 : 1) * render.src.ratio) + 0.5);
+    sdl.srcAspect.x = aspect_ratio_x>0?aspect_ratio_x:(int)(render.src.width * (render.src.dblw ? 2 : 1));
+    sdl.srcAspect.y = aspect_ratio_y>0?aspect_ratio_y:(int)floor((render.src.height * (render.src.dblh ? 2 : 1) * render.src.ratio) + 0.5);
     sdl.srcAspect.xToY = (double)sdl.srcAspect.x / sdl.srcAspect.y;
     sdl.srcAspect.yToX = (double)sdl.srcAspect.y / sdl.srcAspect.x;
     LOG_MSG("Aspect ratio: %u x %u  xToY=%.3f yToX=%.3f",sdl.srcAspect.x,sdl.srcAspect.y,sdl.srcAspect.xToY,sdl.srcAspect.yToX);
@@ -1052,7 +1061,7 @@ extern const char *scaler_menu_opts[][2];
 void RENDER_UpdateScalerMenu(void) {
     const std::string scaler = RENDER_GetScaler();
 
-    mainMenu.get_item("scaler_forced").check(render.scale.forced).refresh_item(mainMenu);
+    mainMenu.get_item("mapper_fscaler").check(render.scale.forced).refresh_item(mainMenu);
     for (size_t i=0;scaler_menu_opts[i][0] != NULL;i++) {
         const std::string name = std::string("scaler_set_") + scaler_menu_opts[i][0];
         mainMenu.get_item(name).check(scaler == scaler_menu_opts[i][0]).refresh_item(mainMenu);
@@ -1181,6 +1190,27 @@ std::string LoadGLShader(Section_prop * section) {
 }
 #endif
 
+void setAspectRatio(Section_prop * section) {
+	char *ratiostr = (char *)section->Get_string("aspect_ratio"), *p = strchr(ratiostr, ':');
+	if (p) {
+		*p = 0;
+		int ratiox = atoi(ratiostr);
+		int ratioy = atoi(p+1);
+		aspect_ratio_x = ratiox>0&&ratioy>0?ratiox:(ratiox==-1&&ratioy==-1?-1:0);
+		aspect_ratio_y = ratiox>0&&ratioy>0?ratioy:(ratiox==-1&&ratioy==-1?-1:0);
+		*p = ':';
+	} else
+		aspect_ratio_x = aspect_ratio_y = 0;
+	aspect_ratio_menu();
+}
+
+void SetScaleForced(bool forced);
+void ForceScaler(bool pressed) {
+    if (!pressed) return;
+    SetScaleForced(!render.scale.forced);
+    return;
+}
+
 void RENDER_Init() {
     Section_prop * section=static_cast<Section_prop *>(control->GetSection("render"));
 
@@ -1195,6 +1225,7 @@ void RENDER_Init() {
 		LOG_MSG("Euro ASCII value has to be between 33 and 255\n");
 		eurAscii = -1;
 	}
+	setAspectRatio(section);
 
 	//Set monochrome mode color and brightness
 	vga.draw.monochrome_pal=MonochromeColor::Green;
@@ -1247,6 +1278,9 @@ void RENDER_Init() {
 	MAPPER_AddHandler(&AspectRatio_mapper_shortcut, MK_nothing, 0, "aspratio", "Fit to aspect ratio", &item);
 	item->set_text("Fit to aspect ratio");
 
+	MAPPER_AddHandler(ForceScaler, MK_nothing, 0, "fscaler", "Force scaler", &item);
+	item->set_text("Force scaler");
+
 	if (machine==MCH_HERC || machine==MCH_MDA) {
 		void HercBlend(bool pressed), CycleHercPal(bool pressed);
 		MAPPER_AddHandler(CycleHercPal,MK_f7,MMOD1,"hercpal","Hercules Palette");
