@@ -26,9 +26,11 @@
 /*****************************************************************************/
 // C++ SDLnet wrapper
 
+#if !defined(__MINGW32__) || defined(__MINGW64_VERSION_MAJOR)
 #define ENET_IMPLEMENTATION
 #include "enet.h"
-
+#endif
+#include "ipx.h"
 #include "logging.h"
 #include "misc_util.h"
 #include "timer.h"
@@ -84,7 +86,9 @@ NETClientSocket *NETClientSocket::NETClientFactory(SocketTypesE socketType,
 	switch (socketType) {
 	case SOCKET_TYPE_TCP: return new TCPClientSocket(destination, port);
 
+#if !defined(__MINGW32__) || defined(__MINGW64_VERSION_MAJOR)
 	case SOCKET_TYPE_ENET: return new ENETClientSocket(destination, port);
+#endif
 
 	default: return nullptr;
 	}
@@ -141,7 +145,9 @@ NETServerSocket *NETServerSocket::NETServerFactory(SocketTypesE socketType,
 	switch (socketType) {
 	case SOCKET_TYPE_TCP: return new TCPServerSocket(port);
 
+#if !defined(__MINGW32__) || defined(__MINGW64_VERSION_MAJOR)
 	case SOCKET_TYPE_ENET: return new ENETServerSocket(port);
+#endif
 
 	default: return nullptr;
 	}
@@ -150,6 +156,7 @@ NETServerSocket *NETServerSocket::NETServerFactory(SocketTypesE socketType,
 
 // --- ENet UDP NET INTERFACE ------------------------------------------------
 
+#if !defined(__MINGW32__) || defined(__MINGW64_VERSION_MAJOR)
 class enet_manager_t {
 public:
 	enet_manager_t()
@@ -522,6 +529,7 @@ void ENETClientSocket::updateState()
 	}
 #endif
 }
+#endif
 
 // --- TCP NET INTERFACE -----------------------------------------------------
 
