@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include "dosbox.h"
 #include "inout.h"
+#include "logging.h"
 #include "vga.h"
 #include "debug.h"
 #include "cpu.h"
@@ -126,12 +127,12 @@ void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
 			5-6	Number of character clocks to delay start of display after Horizontal
 				Retrace.
 			7	bit 5 of the End Horizontal Blanking count (See 3d4h index 3 bit 0-4)
-		*/	
+		*/
 		break;
 	case 0x06: /* Vertical Total Register */
 		if (crtc(read_only)) break;
 		if (val != crtc(vertical_total)) {
-			crtc(vertical_total)=(uint8_t)val;	
+			crtc(vertical_total)=(uint8_t)val;
 			VGA_StartResize();
 		}
 		/*	0-7	Lower 8 bits of the Vertical Total. Bit 8 is found in 3d4h index 7
@@ -240,6 +241,8 @@ void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
 		vga.config.cursor_start&=0xff00ff;
 		vga.config.cursor_start|=val << 8;
 		/*	0-7  Upper 8 bits of the address of the cursor */
+		/* S3 86C928 emulation: In "enhanced mode" (256-color SVGA), this register is also used to hold the hardware cursor foreground
+		 * color index [http://hackipedia.org/browse.cgi/Computer/Platform/PC%2c%20IBM%20compatible/Video/VGA/SVGA/S3%20Graphics%2c%20Ltd/S3%2086C928%20GUI%20Accelerator%20%281992%2d09%29%2epdf] */
 		break;
 	case 0x0F:	/* Cursor Location Low Register */
 //TODO update cursor on screen
@@ -247,10 +250,12 @@ void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
 		vga.config.cursor_start&=0xffff00;
 		vga.config.cursor_start|=val;
 		/*	0-7  Lower 8 bits of the address of the cursor */
+		/* S3 86C928 emulation: In "enhanced mode" (256-color SVGA), this register is also used to hold the hardware cursor background
+		 * color index [http://hackipedia.org/browse.cgi/Computer/Platform/PC%2c%20IBM%20compatible/Video/VGA/SVGA/S3%20Graphics%2c%20Ltd/S3%2086C928%20GUI%20Accelerator%20%281992%2d09%29%2epdf] */
 		break;
 	case 0x10:	/* Vertical Retrace Start Register */
 		crtc(vertical_retrace_start)=(uint8_t)val;
-		/*	
+		/*
 			0-7	Lower 8 bits of Vertical Retrace Start. Vertical Retrace starts when
 			the line counter reaches this value. Bit 8 is found in 3d4h index 7
 			bit 2. Bit 9 is found in 3d4h index 7 bit 7.
@@ -258,7 +263,7 @@ void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
 		break;
 	case 0x11:	/* Vertical Retrace End Register */
 		crtc(vertical_retrace_end)=(uint8_t)val;
-		
+
 		if (IS_EGAVGA_ARCH && !(val & 0x10)) {
 			vga.draw.vret_triggered=false;
 			if (GCC_UNLIKELY(machine==MCH_EGA)) PIC_DeActivateIRQ(9);
@@ -321,7 +326,7 @@ void vga_write_p3d5(Bitu port,Bitu val,Bitu iolen) {
 			crtc(start_vertical_blanking)=(uint8_t)val;
 			VGA_StartResize();
 		}
-		/* 
+		/*
 			0-7	Lower 8 bits of Vertical Blank Start. Vertical blanking starts when
 				the line counter reaches this value. Bit 8 is found in 3d4h index 7
 				bit 3.
@@ -416,7 +421,7 @@ Bitu vga_read_p3d5x(Bitu port,Bitu iolen) {
 	case 0x05:	/* End Horizontal Retrace Register */
 		return crtc(end_horizontal_retrace);
 	case 0x06: /* Vertical Total Register */
-		return crtc(vertical_total);	
+		return crtc(vertical_total);
 	case 0x07:	/* Overflow Register */
 		return crtc(overflow);
 	case 0x08:	/* Preset Row Scan Register */
