@@ -336,7 +336,12 @@ void SVGA_S3_WriteCRTC(Bitu reg,Bitu val,Bitu iolen) {
                 index 9 bit 6.
         */
     case 0x63:  /* Extended Control Register CR63 */
-        if (s3Card == S3_86C928 || s3Card == S3_Vision864 || s3Card == S3_Vision868) return; /* not mentioned in datasheet, does not exist */
+        if (s3Card == S3_86C928 || 
+			s3Card == S3_Vision864 ||
+			s3Card == S3_Vision868 ||
+			s3Card == S3_Vision964 ||
+			s3Card == S3_Vision968)
+		return; /* not mentioned in datasheet, does not exist */
         if (s3Card >= S3_ViRGE && ((val ^ vga.s3.reg_63) & 2u/*RST*/)) SD3_Reset(!!(val & 2u));
         vga.s3.reg_63 = (uint8_t)val;
         break;
@@ -436,6 +441,8 @@ Bitu SVGA_S3_ReadCRTC( Bitu reg, Bitu iolen) {
                 return 0x00; // not mentioned in datasheet, does not exist
             case S3_Vision864:
             case S3_Vision868:
+            case S3_Vision964:
+            case S3_Vision968:
             case S3_Trio32:
             case S3_Trio64:
             case S3_Trio64V:
@@ -462,6 +469,10 @@ Bitu SVGA_S3_ReadCRTC( Bitu reg, Bitu iolen) {
                 return 0xC0; // Vision864, 0x88C0 or 0x88C1
             case S3_Vision868:
                 return 0x80; // Vision868, 0x8880 or 0x8881. S3 didn't list this in their datasheet, but Windows 95 INF files listed it anyway
+            case S3_Vision964:
+                return 0xd0; // Vision964, 0x88d0, 0x88d1, 0x88d2 or 0x88d3
+            case S3_Vision968:
+                return 0xf0; // Vision968, 0x88f0, 0x88f1, 0x88f2 or 0x88f3
             case S3_Trio32:
                 return 0x10; // Trio32. 0x8810 or 0x8811
             case S3_Trio64:
@@ -558,7 +569,12 @@ Bitu SVGA_S3_ReadCRTC( Bitu reg, Bitu iolen) {
     case 0x5e:  /* Extended Vertical Overflow */
         return vga.s3.ex_ver_overflow;
     case 0x63:  /* Extended Control Register CR63 */
-        if (s3Card == S3_86C928 || s3Card == S3_Vision864 || s3Card == S3_Vision868) return 0x00; /* not mentioned in datasheet, does not exist */
+        if (s3Card == S3_86C928 ||
+			s3Card == S3_Vision864 ||
+			s3Card == S3_Vision868 ||
+			s3Card == S3_Vision964 ||
+			s3Card == S3_Vision968)
+		return 0x00; /* not mentioned in datasheet, does not exist */
         return vga.s3.reg_63;
     case 0x67:  /* Extended Miscellaneous Control 2 */
         return vga.s3.misc_control_2;
