@@ -34,6 +34,10 @@
 #include "voodoo_opengl.h"
 #include "sdlmain.h"
 
+bool Voodoo_OGL_Active() {
+	return (v != NULL && v->clock_enabled && v->output_on);
+}
+
 #if C_OPENGL
 
 #if defined(_MSC_VER)
@@ -96,10 +100,6 @@ int Voodoo_OGL_GetHeight() {
 		return 0;
 }
 
-bool Voodoo_OGL_Active() {
-	return (v != NULL && v->clock_enabled && v->output_on);
-}
-
 static void ogl_get_depth(voodoo_state* VV, INT32 ITERZ, INT64 ITERW, INT32 *depthval, INT32 *out_wfloat)
 {
 	INT32 wfloat;
@@ -1683,6 +1683,11 @@ void voodoo_ogl_reset_videomode(void) {
     DOSBox_SetMenu();
 #endif
 
+    bool isfs = false;
+    if (GFX_IsFullscreen()) {
+        isfs = true;
+        GFX_SwitchFullScreen();
+    }
     GFX_PreventFullscreen(true);
 
 	last_clear_color=0;
@@ -1740,8 +1745,8 @@ void voodoo_ogl_reset_videomode(void) {
     GFX_ReleaseMouse();
     GFX_ForceFullscreenExit();
 
-	if (new_height<v->fbi.height) new_height = v->fbi.height;
-	if (new_width < v->fbi.width) new_width = v->fbi.width;
+	/*if (new_height<v->fbi.height) */new_height = v->fbi.height;
+	/*if (new_width < v->fbi.width) */new_width = v->fbi.width;
 
 	if (GFX_LazyFullscreenRequested()) GFX_SwitchFullscreenNoReset();
 
@@ -1787,6 +1792,14 @@ void voodoo_ogl_reset_videomode(void) {
 	}
 #endif
 
+    if (isfs) {
+#if defined(C_SDL2)
+        SDL_MaximizeWindow(sdl.window);
+#elif defined(WIN32)
+        ShowWindow(GetHWND(), SW_MAXIMIZE);
+#endif
+    }
+
     v->ogl_dimchange = true;
 
 	glViewport( 0, 0, new_width, new_height );
