@@ -82,10 +82,18 @@ Bitu INT10_Handler(void) {
 				mode = 0x70;
 				enum DOSV_VTEXT_MODE vtext_mode = DOSV_GetVtextMode((reg_al == 0x70) ? 0 : 1);
 				if(vtext_mode == DOSV_VTEXT_XGA || vtext_mode == DOSV_VTEXT_XGA_24) {
-					INT10_SetVideoMode(0x37);
+					if(svgaCard == SVGA_TsengET4K) {
+						INT10_SetVideoMode(0x37);
+					} else {
+						INT10_SetVideoMode(0x104);
+					}
 					INT10_SetDOSVModeVtext(mode, vtext_mode);
 				} else if(vtext_mode == DOSV_VTEXT_SXGA || vtext_mode == DOSV_VTEXT_SXGA_24) {
-					INT10_SetVideoMode(0x3d);
+					if(svgaCard == SVGA_TsengET4K) {
+						INT10_SetVideoMode(0x3d);
+					} else {
+						INT10_SetVideoMode(0x106);
+					}
 					INT10_SetDOSVModeVtext(mode, vtext_mode);
 				} else if(vtext_mode == DOSV_VTEXT_SVGA) {
 					INT10_SetVideoMode((svgaCard == SVGA_TsengET3K) ? 0x29 : (svgaCard == SVGA_ParadisePVGA1A) ? 0x58 : 0x6a);
@@ -1195,7 +1203,7 @@ static void INT10_Seg40Init(void) {
 #if C_DEBUG
 	if (control->opt_display2)
 		real_writeb(BIOSMEM_SEG,BIOSMEM_MODESET_CTL,0x10|(DISP2_Active()?0:1));
-    else
+	else
 #endif
 	if (IS_EGAVGA_ARCH) {
 		real_writeb(BIOSMEM_SEG,BIOSMEM_MODESET_CTL,0x51); // why is display switching enabled (bit 6) ?
@@ -1253,6 +1261,7 @@ extern bool VIDEO_BIOS_disable;
 extern Bitu BIOS_VIDEO_TABLE_LOCATION;
 extern Bitu BIOS_VIDEO_TABLE_SIZE;
 
+Bitu ROMBIOS_GetMemory(Bitu bytes,const char *who,Bitu alignment,Bitu must_be_at);
 bool ROMBIOS_FreeMemory(Bitu phys);
 Bitu RealToPhys(Bitu x);
 
@@ -1638,6 +1647,18 @@ void INT10_Startup(Section *sec) {
                 E_Exit("VGA BIOS size too small %u > %u",(unsigned int)int10.rom.used,(unsigned int)VGA_BIOS_Size);
         }
 
+	/* If machine=vgaonly, make a dummy stub routine at 0xF000:0xF065 if the user is using the stock IBM VGA ROM BIOS.
+	 * It calls INT 42h for unknown functions. INT 42h is NOT the previous INT 10h handler but is a hardcoded address
+	 * specific to VGA-based IBM PCs of the time period. */
+	if (IS_VGA_ARCH && svgaCard == SVGA_None && VGA_BIOS_use_rom) {
+		LOG(LOG_MISC,LOG_DEBUG)("Creating dummy IBM BIOS INT 10h stub for VGA BIOS ROM image at F000:F065");
+		// TODO: This may have to identify the ROM image by file name in case other ROM images have other
+		//       specific requirements or hardcoded addresses.
+		PhysPt base = ROMBIOS_GetMemory(3,"IBM BIOS dummy INT 10h stub for VGA BIOS",/*align*/1,0xFF065); /* 0xF000:0xF065 */
+		phys_writew(base+0,0xC031);		// XOR AX,AX
+		phys_writeb(base+2,0xCF);		// IRET
+	}
+
         /* NTS: Uh, this does seem bass-ackwards... INT 10h making the VGA BIOS appear. Can we refactor this a bit? */
         if (VGA_BIOS_Size > 0) {
             LOG(LOG_MISC,LOG_DEBUG)("VGA BIOS occupies segment 0x%04x-0x%04x",(int)VGA_BIOS_SEG,(int)VGA_BIOS_SEG_END-1);
