@@ -1359,6 +1359,9 @@ static Bitu dosv_cursor_16[] = {
 	0, 3, 5, 7, 9, 11, 13, 15
 };
 
+uint8_t StartBankSelect(Bitu &off);
+uint8_t CheckBankSelect(uint8_t select, Bitu &off);
+
 void DOSV_CursorXor24(Bitu x, Bitu y, Bitu start, Bitu end)
 {
 	IO_Write(0x3ce, 0x05); IO_Write(0x3cf, 0x03);
@@ -1368,140 +1371,35 @@ void DOSV_CursorXor24(Bitu x, Bitu y, Bitu start, Bitu end)
 	Bitu width = (real_readw(BIOSMEM_SEG, BIOSMEM_NB_COLS) == 85) ? 128 : 160;
 	volatile uint8_t dummy;
 	Bitu off = (y + start) * width + (x * 12) / 8;
-	uint8_t select = 0;
-	if(svgaCard == SVGA_TsengET4K) {
-		if(off >= 0x20000) {
-			select = 0x22;
-			off -= 0x20000;
-		} else if(off >= 0x10000) {
-			select = 0x11;
-			off -= 0x10000;
-		} else {
-			select = 0x00;
-		}
-		IO_Write(0x3cd, select);
-	}
+	uint8_t select = StartBankSelect(off);
 	while(start <= end) {
 		if(dosv_cursor_stat == 2) {
-			if(x & 1) {
+			uint8_t cursor_data[2][4] = {{ 0xff, 0xff, 0xff, 0x00 }, { 0x0f, 0xff, 0xff, 0xf0 }};
+			for(uint8_t i = 0 ; i < 4 ; i++) {
 				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0x0f);
+				real_writeb(0xa000, off, cursor_data[x & 1][i]);
 				off++;
-				if(off >= 0x10000) {
-					if(select == 0x00) {
-						select = 0x11;
-					} else if(select == 0x11) {
-						select = 0x22;
-					}
-					IO_Write(0x3cd, select);
-					off -= 0x10000;
-				}
-				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0xff);
-				off++;
-				if(off >= 0x10000) {
-					if(select == 0x00) {
-						select = 0x11;
-					} else if(select == 0x11) {
-						select = 0x22;
-					}
-					IO_Write(0x3cd, select);
-					off -= 0x10000;
-				}
-				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0xff);
-				off++;
-				if(off >= 0x10000) {
-					if(select == 0x00) {
-						select = 0x11;
-					} else if(select == 0x11) {
-						select = 0x22;
-					}
-					IO_Write(0x3cd, select);
-					off -= 0x10000;
-				}
-				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0xf0);
-				off += width - 3;
-			} else {
-				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0xff);
-				off++;
-				if(off >= 0x10000) {
-					if(select == 0x00) {
-						select = 0x11;
-					} else if(select == 0x11) {
-						select = 0x22;
-					}
-					IO_Write(0x3cd, select);
-					off -= 0x10000;
-				}
-				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0xff);
-				off++;
-				if(off >= 0x10000) {
-					if(select == 0x00) {
-						select = 0x11;
-					} else if(select == 0x11) {
-						select = 0x22;
-					}
-					IO_Write(0x3cd, select);
-					off -= 0x10000;
-				}
-				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0xff);
-				off += width - 2;
+				select = CheckBankSelect(select, off);
 			}
+			off += width - 4;
 		} else {
-			if(x & 1) {
+			uint8_t cursor_data[2][2] = { { 0xff, 0xf0 }, { 0x0f, 0xff }};
+			for(uint8_t i = 0 ; i < 2 ; i++) {
 				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0x0f);
+				real_writeb(0xa000, off, cursor_data[x & 1][i]);
 				off++;
-				if(off >= 0x10000) {
-					if(select == 0x00) {
-						select = 0x11;
-					} else if(select == 0x11) {
-						select = 0x22;
-					}
-					IO_Write(0x3cd, select);
-					off -= 0x10000;
-				}
-				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0xff);
-			} else {
-				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0xff);
-				off++;
-				if(off >= 0x10000) {
-					if(select == 0x00) {
-						select = 0x11;
-					} else if(select == 0x11) {
-						select = 0x22;
-					}
-					IO_Write(0x3cd, select);
-					off -= 0x10000;
-				}
-				dummy = real_readb(0xa000, off);
-				real_writeb(0xa000, off, 0xf0);
-			}
-			off += width - 1;
-		}
-		if(svgaCard == SVGA_TsengET4K) {
-			if(off >= 0x10000) {
-				if(select == 0x00) {
-					select = 0x11;
-				} else if(select == 0x11) {
-					select = 0x22;
-				}
-				IO_Write(0x3cd, select);
-				off -= 0x10000;
+				select = CheckBankSelect(select, off);
 			}
+			off += width - 2;
 		}
+		select = CheckBankSelect(select, off);
 		start++;
 	}
 	IO_Write(0x3ce, 0x03); IO_Write(0x3cf, 0x00);
 	if(svgaCard == SVGA_TsengET4K) {
 		IO_Write(0x3cd, 0);
+	} else if(svgaCard == SVGA_S3Trio) {
+		IO_Write(0x3d4, 0x6a); IO_Write(0x3d5, 0);
 	}
 }
 
@@ -1531,55 +1429,27 @@ void DOSV_CursorXor(Bitu x, Bitu y)
 
 		volatile uint8_t dummy;
 		Bitu off = (y + start) * width + x;
-		uint8_t select = 0;
-		if(svgaCard == SVGA_TsengET4K) {
-			if(off >= 0x20000) {
-				select = 0x22;
-				off -= 0x20000;
-			} else if(off >= 0x10000) {
-				select = 0x11;
-				off -= 0x10000;
-			} else {
-				select = 0x00;
-			}
-			IO_Write(0x3cd, select);
-		}
+		uint8_t select = StartBankSelect(off);
 		while(start <= end) {
 			dummy = real_readb(0xa000, off);
 			real_writeb(0xa000, off, 0xff);
 			if(dosv_cursor_stat == 2) {
 				off++;
-				if(off >= 0x10000) {
-					if(select == 0x00) {
-						select = 0x11;
-					} else if(select == 0x11) {
-						select = 0x22;
-					}
-					IO_Write(0x3cd, select);
-					off -= 0x10000;
-				}
+				select = CheckBankSelect(select, off);
 				dummy = real_readb(0xa000, off);
 				real_writeb(0xa000, off, 0xff);
 				off += width - 1;
 			} else {
 				off += width;
 			}
-			if(svgaCard == SVGA_TsengET4K) {
-				if(off >= 0x10000) {
-					if(select == 0x00) {
-						select = 0x11;
-					} else if(select == 0x11) {
-						select = 0x22;
-					}
-					IO_Write(0x3cd, select);
-					off -= 0x10000;
-				}
-			}
+			select = CheckBankSelect(select, off);
 			start++;
 		}
 		IO_Write(0x3ce, 0x03); IO_Write(0x3cf, 0x00);
 		if(svgaCard == SVGA_TsengET4K) {
 			IO_Write(0x3cd, 0);
+		} else if(svgaCard == SVGA_S3Trio) {
+			IO_Write(0x3d4, 0x6a); IO_Write(0x3d5, 0);
 		}
 	}
 }
@@ -1689,7 +1559,7 @@ enum DOSV_VTEXT_MODE DOSV_GetVtextMode(Bitu no)
 
 enum DOSV_VTEXT_MODE DOSV_StringVtextMode(std::string vtext)
 {
-	if(svgaCard == SVGA_TsengET4K) {
+	if(svgaCard == SVGA_TsengET4K || svgaCard == SVGA_S3Trio) {
 		if(vtext == "xga") {
 			return DOSV_VTEXT_XGA;
 		} else if(vtext == "xga24") {
@@ -1751,8 +1621,8 @@ static uint16_t j3_cursor_x;
 static uint16_t j3_cursor_y;
 static Bitu j3_text_color;
 static Bitu j3_back_color;
-static uint16_t j3_font_offset;
 static uint16_t j3_machine_code = 0;
+uint16_t j3_font_offset = 0xca00;
 
 static uint16_t jis2shift(uint16_t jis)
 {
@@ -2198,7 +2068,7 @@ uint16_t J3_GetMachineCode() {
 	return j3_machine_code;
 }
 
-void J3_SetType(std::string type) {
+void J3_SetType(std::string type, std::string back, std::string text) {
 	if(type != "default") {
 		j3_machine_code = ConvHexWord((char *)type.c_str());
 	}
@@ -2211,8 +2081,16 @@ void J3_SetType(std::string type) {
 			break;
 		}
 	}
-	j3_back_color = back_color_list[j3_color];
-	j3_text_color = text_color_list[j3_color];
+	if(back.empty()) {
+		j3_back_color = back_color_list[j3_color];
+	} else {
+		j3_back_color = ConvHexWord((char *)back.c_str());
+	}
+	if(text.empty()) {
+		j3_text_color = text_color_list[j3_color];
+	} else {
+		j3_text_color = ConvHexWord((char *)text.c_str());
+	}
 }
 
 void J3_GetPalette(uint8_t no, uint8_t &r, uint8_t &g, uint8_t &b)
